import '../uibit.dart';

class KSliderController extends TController {
  final double initialValue;
  double _value = 0;

  KSliderController({this.initialValue}) {
    _value = initialValue ?? 0.0;
  }

  double get value => _value;

  set value(double value) {
    if (value == _value) return;
    _value = value;
    notifyListeners();
  }
}

class KSlider extends StatefulWidget {
  final Color color;
  final Axis axis;
  final Duration duration;
  final double value;
  final double size;
  final double paint;
  final KSliderController controller;
  final ValueChanged<double> onChanged;
  final ValueChanged<double> onStart;
  final ValueChanged<double> onEnd;

  KSlider({this.color, this.axis, this.duration, this.value = 0.0, this.controller, this.onChanged, this.size, this.onStart, this.onEnd, this.paint}) : assert(value != null);

  @override
  KSliderState createState() => KSliderState();
}

class KSliderState extends State<KSlider> {
  KSliderController controller;
  double _pos = 0.0;
  RenderBox _box;

  get getDuration => widget.duration ?? Duration(milliseconds: 100);

  get getAxis => widget.axis ?? Axis.horizontal;

  @override
  void initState() {
    super.initState();
    controller = widget.controller ?? KSliderController(initialValue: widget.value);
    controller.addListener(() {
      setState(() {});
    });
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }

  _updateDragPosition(Offset start) {
    _box = _box ?? context.findRenderObject();
    Offset val = _box.globalToLocal(start);
    _pos = val.dy;
    if (getAxis == Axis.horizontal) _pos = val.dx;
    controller.value = _pos / _box.size.longestSide;
  }

  _onDragStart(DragStartDetails start) {
    if (widget.onStart != null) widget.onStart(controller.value);
    _updateDragPosition(start.globalPosition);
  }

  _onDragUpdate(DragUpdateDetails update) {
    _updateDragPosition(update.globalPosition);
    if (widget.onChanged != null) widget.onChanged(controller.value);
  }

  _onDragEnd(DragEndDetails details) {
    if (widget.onEnd != null) widget.onEnd(controller.value);
  }

  double get minSize=>widget.size ?? kSize.slider;

  @override
  Widget build(BuildContext context) {
    if (Axis.vertical == getAxis) {
      return GestureDetector(
        onVerticalDragStart: _onDragStart,
        onVerticalDragUpdate: _onDragUpdate,
        onVerticalDragEnd: _onDragEnd,
        child: Container(
          width: minSize,
          height: double.infinity,
          child: AnimatedContainer(
            duration: Duration(milliseconds: 100),
            curve: Curves.fastOutSlowIn,
            width: minSize,
            alignment: Alignment.topRight,
            decoration: _decoration(),
            child: FractionallySizedBox(
              heightFactor: controller.value.clamp(0.001, 0.999),
              alignment: Alignment.topCenter,
              child: AnimatedContainer(
                duration: getDuration,
                decoration: _decoration(),
              ),
            ),
          ),
        ),
      );
    }
    return GestureDetector(
      onHorizontalDragStart: _onDragStart,
      onHorizontalDragUpdate: _onDragUpdate,
      onHorizontalDragEnd: _onDragEnd,
      child: Container(
        width: double.infinity,
        height: widget.size ?? kSize.slider,
        decoration: _decoration(),
        child: FractionallySizedBox(
          alignment: Alignment.centerLeft,
          widthFactor: controller.value.clamp(0.001, 0.999),
          child: AnimatedContainer(
            duration: getDuration,
            decoration: _decoration(),
          ),
        ),
      ),
    );
  }

  _decoration() {
    return BoxDecoration(color: widget.color ?? kColor.opa(0.1), borderRadius: BorderRadius.circular(minSize*2));
  }
}
