
import '../uibit.dart';

enum PopUpStatus { hide, up, down }

Future<T> showDialogCustom<T>({
  @required BuildContext context,
  @required Widget child,
}) {
  return showGeneralDialog<T>(
    context: context,
    pageBuilder: (BuildContext buildContext, Animation<double> animation, Animation<double> secondaryAnimation) {
      return FadeTransition(
        opacity: Tween(begin: 0.0, end: 1.0).animate(animation),
        child: child,
      );
    },
    barrierDismissible: true,
    barrierLabel: MaterialLocalizations.of(context).modalBarrierDismissLabel,
    barrierColor: Color(0x01FFFFFF),
    transitionDuration: Duration(milliseconds: 200),
    transitionBuilder: (BuildContext context, Animation<double> animation, Animation<double> secondaryAnimation, Widget child) {
      return FadeTransition(
        opacity: Tween(begin: 0.0, end: 1.0).animate(animation),
        child: child,
      );
    },
    useRootNavigator: true,
  );
}

class PopUpController extends TController<Function(PopUpStatus), PopUpStatus> {
  final double initialTop;
  final bool scroll;
  final Widget Function() ground;

  PopUpController({this.initialTop, this.scroll = true, this.ground}) {
    _kMinTop = initialTop ?? 50;
  }

  PopUpStatus status = PopUpStatus.hide;

  double get kMinTop => _kMinTop;
  double _kMinTop = 0;

  set kMinTop(double d) {
    _kMinTop = d;
    onSwipeEnd(null);
  }

  onPopUp() {
    if (!scroll) return;
    status = PopUpStatus.up;
    notifyTListeners(status);
  }

  onPopDown() {
    status = PopUpStatus.down;
    notifyTListeners(status);
  }

  set position(double s) {
    _position = s;
    notifyListeners();
  }

  double get position => _position;
  double _position = 0;

  double _lastTapDy = 0;

  _onTapDown(TapDownDetails d) {
    _lastTapDy = d.globalPosition.dy;
  }

  onTap() {
    if (ground != null) return;
    if (position > _lastTapDy) onPopDown();
  }

  onSwipeUpdate(DragUpdateDetails update) {
    if (!scroll || update == null) return;
    if (position > kSize.kTopPadding) position += update.primaryDelta;
  }

  onSwipeEnd(DragEndDetails details) {
    if (kSize.height - position < kSize.height / 2) {
      onPopDown();
    } else {
      onPopUp();
    }
  }

  background() {
    if (ground != null) return ground();
    return Center();
  }
}

class PopUp extends StatelessWidget {
  final Widget child;
  final PopUpController controller;

  const PopUp({Key key, this.child, this.controller}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.transparent,
      body: InkWell(
        onTap: controller.onTap,
        onTapDown: controller._onTapDown,
        child: PopUpBody(
          controller: controller,
          child: child,
        ),
      ),
    );
  }
}

class PopUpBody extends StatefulWidget {
  final PopUpController controller;
  final Widget child;

  const PopUpBody({
    Key key,
    @required this.child,
    @required this.controller,
  })  : assert(controller != null),
        assert(child != null),
        super(key: key);

  @override
  PopUpBodyState createState() => PopUpBodyState();
}

class PopUpBodyState extends State<PopUpBody> with TickerProviderStateMixin {
  Animation<double> animation;
  AnimController animationController;

  PopUpController get controller => widget.controller;

  @override
  void initState() {
    super.initState();
    animationController = AnimController(duration: Duration(milliseconds: 350), vsync: this);
    animationController.addStatusListener((status) {
      if (controller.status == PopUpStatus.down && status == AnimationStatus.completed) Navigator.of(context).pop();
    });

    animationController.addListener(() {
      controller.position = animation.value;
    });


    final w2 = controller.initialTop ?? kSize.height / 5 * 3;

    final curveAnim = CurvedAnimation(parent: animationController, curve: Curves.easeInOut);

    animation = Tween<double>(begin: kSize.height, end: w2).animate(curveAnim);
    animationController.reset();
    animationController.forward();

    controller.addTListener((status) {
      if (status == PopUpStatus.hide) return;
      if (status == PopUpStatus.up) animation = Tween<double>(begin: controller.position, end: controller.kMinTop).animate(curveAnim);
      if (status == PopUpStatus.down) animation = Tween<double>(begin: controller.position, end: kSize.height).animate(curveAnim);
      animationController.reset();
      animationController.forward();
    });
    controller.addListener(_registerListener);
  }

  _registerListener() {
    setState(() {});
  }

  @override
  void dispose() {
    controller.removeListener(_registerListener);
    animationController.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: Colors.transparent,
      child: Stack(
        children: <Widget>[
          controller.background(),
          Positioned(
            top: controller.position,
            left: 0,
            child: widget.child,
          ),
        ],
      ),
    );
  }
}
